# -*- coding: utf-8 -*-


import sys
import cv2
import numpy as np

def gaussian_curve(x, y, sigma):
    try:
        return np.e ** (-(x*x + y*y)/(2*sigma*sigma))
    except ZeroDivisionError:
        print "Erro: Sigma não pode ser 0."
        sys.exit(128)


def generate_gaussian(sigma, window_size):
    if(window_size%2 == 0):
        print("Apeans janelas ímpares, por favor.")
        sys.exit(4)
    half_window_size = window_size/2
    x_pos = np.array([[y for x in range(-half_window_size, half_window_size+1)] for y in range(-half_window_size, half_window_size+1)])
    y_pos = np.array([[x for x in range(-half_window_size, half_window_size+1)] for y in range(-half_window_size, half_window_size+1)])
    gauss = np.vectorize(gaussian_curve)
    return gauss(x_pos, y_pos, sigma) * 1/(2*np.pi*sigma*sigma)

def calculate_median(img, x, y, start_x, end_x, start_y, end_y, ch):
    window = img[start_y:end_y, start_x:end_x,ch]
    median_index = window.size/2
    new_value = np.sort(window.flatten())[median_index]
    return new_value

def mediam_filter(img, wsize):
    new_img = np.copy(img)
    height, width, channels = img.shape[:3]
    wsize_par = 0 if wsize%2 == 1 else 1
    for ch in range(0, channels):
        # Lidando com o "miolo" da imagem primeiro
        for y in range(wsize/2-wsize_par, height):
            start_y = y - wsize/2 + wsize_par
            end_y = y + wsize/2 + 1
            for x in range(wsize/2-wsize_par, width):
                start_x = x - wsize/2 + wsize_par
                end_x = x + wsize/2 + 1
                new_img[y,x,ch] = calculate_median(img, x, y, start_x, end_x, start_y, end_y, ch)

        #Lidando com os cantos
        #Etapa 1: Lidar com o quadrado entre (0,0) e (wsize-1,wsize-1)
        for y in range(0, wsize/2-wsize_par):
            end_y = y + wsize/2 + 1
            for x in range(0, wsize/2-wsize_par):
                end_x = x + wsize/2 + 1
                calculate_median(img, x, y, 0, end_x, 0, end_y, ch)
            
            #Etapa 2: Lidar com o retangulo entre (wsize, 0) e (width-1, wsize-1)
            for x in range(wsize/2-wsize_par, width):
                start_x = x - wsize/2 + wsize_par 
                end_x = x + wsize/2
                calculate_median(img, x, y,  start_x, end_x, 0, end_y, ch)

        ##Etapa 3: Lidar com o retangulo entre (0, wsize) e (wsize-1, height-1)
        for y in range(wsize/2-wsize_par, height):
            start_y = y - wsize/2 + wsize_par
            end_y = y + wsize/2
            for x in range(0, wsize):
                end_x = x + wsize/2
                calculate_median(img, x, y,  0, end_x, start_y, end_y, ch)

    return new_img




# Início da "main".

exercicio = 0
try:
    exercicio = int(sys.argv[1] if len(sys.argv) > 1 else raw_input("Número do exercício a ser executado [1-2]:"))
except ValueError:
    print "Input inválido. O número do exercício deve ser um número válido."
    sys.exit(1)

path_image = sys.argv[2] if len(sys.argv) > 2 else raw_input("Path para a imagem: ")
image = cv2.imread(path_image)
if image is None:
    print "Não foi possível abrir a imagem."
    sys.exit(2)

if exercicio == 1:
    print "Ex 1 - Gaussian filter"
    sigma = 0.0
    wsize = 0
    try: 
        sigma = float(sys.argv[3] if len(sys.argv) > 3 else raw_input("Sigma: "))
        wsize = int(sys.argv[4] if len(sys.argv) > 4 else raw_input("Window size: "))
    except ValueError:
        print("Input inválido. Sigma deve ser um float válido, wsize deve ser um inteiro válido.")
        sys.exit(3)
    kernel = generate_gaussian(sigma, wsize)
    print "sum: " + str(np.sum(kernel))
    kernel /= np.sum(kernel)
    print kernel
    print "new sum: " + str(np.sum(kernel))
    new_img = cv2.filter2D(image, -1, kernel)
    cmp_img = np.hstack((image, new_img))
    cv2.imshow("result", cmp_img)
    cv2.waitKey()
    
elif exercicio == 2:
    print "Ex 2 - Median filter"
    wsize = 0
    try:
        wsize = int(sys.argv[3] if len(sys.argv) > 3 else raw_input("Window size: "))
    except ValueError:
        print("Input inválido. Window size deve ser um inteiro válido.")
        sys.exit(3)
    new_img = mediam_filter(image, wsize)
    cmp_img = np.hstack((image, new_img))
    cv2.imshow("result", cmp_img)
    cv2.waitKey()
    
else:
    print "Exercício não existente."
    sys.exit(255)
